/*Android Csound6 Simulation of FROM A DREAM (2019)
Arthur B. Hunkins

Requires installation of a recent Csound for Android app, v 6.13/4 or higher, and at least Android OS 6.0. (For earlier versions of Android Csound, including the "alternative" Android Csound apps found at: http://www.arthunkins.com/Android_Csound_Apps.htm , see DreamAndroidOlder.csd.) Transfer this file via USB cable to the Music folder of your device. Run DreamAndroid.csd from the app.

This simulation follows all the "rules" for live performance by two performers alternating complete patterns. (The free choices among pitches/rests are here made randomly.) Tempo is steady and dynamics unchanging.

A number of performance parameters (including INSTRUMENT and DURATION) are set in the #define statements at the beginning of this file, and may be modified by the user in any text editor. (The entire running time for Csound [one day] may also be changed at the very bottom of the file.)

Length of performance is determined by which of the following occurs first *following* the required 16 patterns: 1) after a pattern in which any bar is completely silent; 2) after the preset # of Patterns has played; 3) after a pattern has exceeded the preset Duration; 4) and finally, when Csound run time elapses.

Notes: 1) The demo Dream(simulation).wav accompanying this .zip archive uses all default settings - 16 pattern repetitions at a tempo of MM180 on the Perry Cook/John ffitch marimba.
2) This .wav is but one random recording of a (very brief) simulated performance. Each running of the .csd will produce a unique result.*/

  
<CsoundSynthesizer>

<CsOptions>
-odac -m0d
</CsOptions>

<CsInstruments>

sr = 44100
ksmps = 128
nchnls = 2

#define INSTRUMENT #-1# ; -1 = marimba, -2 = vibraphone, -3 = cowbells - these (negative) instruments are physical models by Perry Cook/John ffitch; 4 = electric piano, 6 = harpsichord, 8 = celesta, 9 = glockenspiel, 12 = another marimba, 13 = xylophone, 14 = tubular bells, 105 = banjo, 107 = koto, 108 = kalimba - the remainder (positive instruments) are from sf_GMbank.sf2  

#define MIDI_BASE #60# ; base MIDI note (C4)
#define TEMPO #180# ; MM per half note ; range = 50 - 240+
#define DURATION #5 * 60# ; max duration (in seconds)
#define PATTERNS #16# ; max # of repetitions ; allowed range = 16 or more
#define EMPHASIS #.5# ; additional emphasis (frequency of event) on main note per bar; allowed range = 0 - 1

gitempo	= 60 / $TEMPO
	seed 0
	if $INSTRUMENT < 0 igoto skip
gisf	sfload "/storage/emulated/0/Music/samples/sf_GMbank.sf2"
;	sfplist gisf
ginstr	sfpreset $INSTRUMENT, 0, gisf, 0
skip:

	instr 1

indx	init 0
giswitch	init 0
indx2	init 1
istop	init 0

clock:
	timout 0, gitempo, contin
	reinit clock
contin:
indx	= indx + 1
	if indx == 17 then
indx	= 1
giswitch	= (giswitch == 0? 1: 0)
indx2	= indx2 + 1
	endif
back:
inote	unirand 3.999 + $EMPHASIS
inote4	unirand 6.999 + $EMPHASIS

	if indx < 5 then
inote	tab_i inote, 2
	if indx == 1 then
isum	= 0
i1	= inote
	if (indx2 < 6) || (indx2 == $PATTERNS)  then
i1	= 0
inote	= 0
	endif
isum	= isum + i1
	elseif indx == 2 then
i2	= inote
	if ((i1 != -1) && (i2 == i1)) goto back
	if indx2 < 5 then
i2	= 4
inote	= 4
	endif
isum	= isum + i2
	elseif indx == 3 then
i3	= inote
	if (i2 != -1) && (i3 == i2) goto back
	if indx2 < 5 then
i3	= 7
inote	= 7
	endif
isum	= isum + i3
	elseif indx == 4 then
	if (i3 != -1) && (inote == i3) goto back
	if (i3 != -1) && (i1 == i3) && (i2 != -1) && (inote == i2) goto back 
	if indx2 < 5 then
inote	= 4
	endif
isum	= isum + inote
istop	= ((isum == -4) && (indx2 > 16)? 1: istop)	
	endif

	elseif (indx > 4) && (indx < 9) then
inote	tab_i inote, 3
	if indx == 5 then
isum2	= 0
i1	= inote
	if (indx2 < 5) || (indx2 == $PATTERNS)  then
i1	= 5
inote	= 5
	endif
isum2	= isum2 + inote
	elseif indx == 6 then
i2	= inote
	if (i1 != -1) && (i2 == i1) goto back
	if indx2 < 4 then
i2	= 9
inote	= 9
	endif
isum2	= isum2 + inote
	elseif indx == 7 then
i3	= inote
	if (i2 != -1) && (i3 == i2) goto back
	if indx2 < 4 then
i3	= 12
inote	= 12
	endif
isum2	= isum2 + inote
	elseif indx == 8 then
	if inote == i3 goto back
	if (i3 != -1) && (inote == i3) goto back
	if (i3 != -1) && (i1 == i3) && (i2 != -1) && (inote == i2) goto back 
	if indx2 < 4 then
inote	= 9
	endif
isum2	= isum2 + inote
istop	= ((isum2 == -4) && (indx2 > 16)? 1: istop)	
	endif

	elseif (indx > 8) && (indx < 13) then
inote	tab_i inote, 4
	if indx == 9 then
isum3	= 0
i1	= inote
	if (indx2 < 4) || (indx2 == $PATTERNS)  then
i1	= 7
inote	= 7
	endif
isum3	= isum3 + inote
	elseif indx == 10 then
i2	= inote
	if (i1 != -1) && (i2 == i1) goto back
	if indx2 < 3 then
i2	= 11
inote	= 11
	endif
isum3	= isum3 + inote
	elseif indx == 11 then
i3	= inote
	if (i2 != -1) && (i3 == i2) goto back
	if indx2 < 3 then
i3	= 14
inote	= 14
	endif
isum3	= isum3 + inote
	elseif indx == 12 then
	if (i3 != -1) && (inote == i3) goto back
	if (i3 != -1) && (i1 == i3) && (i2 != -1) && (inote == i2) goto back 
	if indx2 < 3 then
inote	= 11
	endif
isum3	= isum3 + inote
istop	= ((isum3 == -4) && (indx2 > 16)? 1: istop)	
	endif

	elseif indx > 12 then
inote	tab_i inote4, 5
	if indx == 13 then
isum4	= 0
i1	= inote
	if (indx2 < 3) || (indx2 == $PATTERNS)  then
i1	= 12
inote	= 12
	endif
isum4	= isum4 + inote
	elseif indx == 14 then
i2	= inote
	if (i1 != -1) && (i2 == i1) goto back
	if (i1 != -1) && (i2 != -1) && (abs(i2 - i1) > 12) goto back
	if indx2 < 2 then
i2	= 16
inote	= 16
	endif
isum4	= isum4 + inote
	elseif indx == 15 then
i3	= inote
	if (i2 != -1) && (i3 == i2) goto back
	if (i2 != -1) && (i3 != -1) && (abs(i3 - i2) > 12) goto back
	if (i2 == -1) && (i3 != -1) && (i1 != -1) && (abs(i3 - i1) > 12) goto back
	if indx2 < 2 then
i3	= 12
inote	= 12
	endif
isum4	= isum4 + inote
	elseif indx == 16 then
	if (i3 != -1) && (inote == i3) goto back
	if (i3 != -1) && (i1 == i3) && (i2 != -1) && (inote == i2) goto back 
	if (i3 != -1) && (inote != -1) && (abs(inote - i3) > 12) goto back
	if (i3 == -1) && (inote != -1) && (i2 != -1) && (abs(inote - i2) > 12) goto back
	if (i2 == -1) && (i3 == -1) && (inote != -1) && (i1 != -1) && (abs(inote - i1) > 12) goto back
	if indx2 < 2 then
inote	= 7
	endif
isum4	= isum4 + inote
istop	= ((isum4 == -4) && (indx2 > 16)? 1: istop)	
	endif
	endif

	if inote > -1 then
ginote	= $MIDI_BASE + inote
gifreq 	= cpsmidinn(ginote)
	event_i "i", 2, 0, 1
	endif
itime	times
	if indx == 16 then
	if (itime > $DURATION) || (istop == 1) || (indx2 == $PATTERNS) then
	turnoff
	endif
	endif 

	endin

	instr 2
	
	if $INSTRUMENT == -1 then
a1	marimba 4000, gifreq, .7, .7, 1, 0, 0, 1, 1, .000000001, .00000000001
	elseif $INSTRUMENT == -2 then
a1	vibes 16000, gifreq, .4, .085, 1, 2, .035, 1, 1
	elseif $INSTRUMENT == -3 then
a1	gogobel 8000, gifreq, 0, .499, 1, 6, .3, 1
	else
kamp	linseg 2.5, .1, 2.5, .9, 0
a1	sfplaym 1, ginote, kamp, 1, ginstr
	endif

	if giswitch == 0 then
	outs a1, a1 * .3
	else
	outs a1 * .3, a1
	endif

	endin

</CsInstruments>
<CsScore>

f1 0 256 1 "/storage/emulated/0/Music/rawwaves/marmstk1.raw" 0 -4 0
f2 0 8 -2 -1 0 4 7 0
f3 0 8 -2 -1 5 9 12 5
f4 0 8 -2 -1 7 11 14 7
f5 0 8 -2 -1 0 4 7 12 16 19 12

i1 0 [60 * 60 * 24] ;  one day max run time

e

</CsScore>
</CsoundSynthesizer>
